from flask import Flask, request, render_template, redirect, url_for, flash, send_file
from flaskext.mysql import MySQL
from flask_wtf.csrf import CSRFProtect, CSRFError
from pymysql.cursors import DictCursor
from werkzeug.utils import secure_filename
import os

from config_db import *

from datetime import datetime, timedelta
from form import DocumentUploadForm as docup
from testku import text_preprocesing
from preprocesing_text import *
from tf_idf import *
from naive_bayes import *
from genplot import *

# development
assets_dir = assets_dir

app = Flask(__name__)

# declare variabel dari config_db.py
database_username = database_username
database_password = database_password
database_ip       = database_ip
database_name     = database_name
secret_key        = secret_key
environment       = env

# config
app.config['SECRET_KEY'] = secret_key
app.config['FLASK_ENV'] = environment
app.config['UPLOAD_FOLDER'] = assets_dir

app.config['MYSQL_DATABASE_USER'] = database_username
app.config['MYSQL_DATABASE_PASSWORD'] = database_password
app.config['MYSQL_DATABASE_DB'] = database_name
app.config['MYSQL_DATABASE_HOST'] = database_ip
app.config['MYSQL_DATABASE_CHARSET'] = "utf8mb4"

app.config['UPLOAD_EXTENSIONS'] = ['.xlsx']

csrf = CSRFProtect(app)
mysql = MySQL(cursorclass=DictCursor)

# peggunaan instan diterapkan global
csrf.init_app(app)
mysql.init_app(app)

# error handling untuk page 404 dan 500
@app.errorhandler(404)
def not_found_error(error):
    return render_template('404.html'), 404

@app.errorhandler(500)
def internal_error(error):
    return render_template('500.html'), 500

@app.errorhandler(CSRFError)
def handle_csrf_error(error):
    return render_template('500.html'), 400

@app.route('/')
@app.route('/index')
def index():
    return render_template('index_sentimen.html')

@app.route('/dashboard')
def home():
    return render_template('index_sentimen.html')

# route sentimen untuk input satuan dari pengguna
# pada testing input manual ini diggunakan dataset 520 dari dataset yang telah dipre-processing
@app.route('/sentimen', methods=['GET','POST'])
def index_sentimen():

    # data diambil dari database MySQL
    query = """select * from dataset_procesed"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    #
    if data:
        df = pd.DataFrame(data)
        # print(df.head())
        df["tweet_list"] = df["tweet_tokens_stemmed"].apply(convert_text_list)
        df["TF_dict"] = df['tweet_list'].apply(calc_TF)
        DF = calc_DF(df["TF_dict"])
        n_document = len(df)
        IDF = calc_IDF(n_document, DF)
        # print(IDF)

        df_kamusdata = pd.DataFrame(list(IDF.items()),columns = ['term','tf'])

        gk = df.groupby('label')
        n_dok = gk.size()
        # print(n_dok)

        df_kamusdata_pos = gk.get_group(1)
        # print(df_kamusdata_pos)
        DF_pos = calc_DF(df_kamusdata_pos["TF_dict"])
        IDF_pos = calc_IDF(n_document, DF_pos)
        df_kamusdata_posnya = pd.DataFrame(list(DF_pos.items()),columns = ['term','tf'])
        df_kamusdata_posnya["idf"] = pd.DataFrame(list(IDF_pos.values()),columns = ['idf'])
        df_kamusdata_posnya["tf_idf_dict"] = df_kamusdata_posnya.tf * df_kamusdata_posnya.idf
        # df_kamusdata_posnya["TF-IDF_dict"].sum()

        df_kamusdata_net = gk.get_group(2)
        # print(df_kamusdata_pos)
        DF_net = calc_DF(df_kamusdata_net["TF_dict"])
        IDF_net = calc_IDF(n_document, DF_net)
        df_kamusdata_netnya = pd.DataFrame(list(DF_net.items()),columns = ['term','tf'])
        df_kamusdata_netnya["idf"] = pd.DataFrame(list(IDF_net.values()),columns = ['idf'])
        df_kamusdata_netnya["tf_idf_dict"] = df_kamusdata_netnya.tf * df_kamusdata_netnya.idf
        # df_kamusdata_posnya["TF-IDF_dict"].sum()
        
        df_kamusdata_neg = gk.get_group(0)
        # print(df_kamusdata_neg)
        DF_neg = calc_DF(df_kamusdata_neg["TF_dict"])
        IDF_neg = calc_IDF(n_document, DF_neg)
        df_kamusdata_negnya = pd.DataFrame(list(DF_neg.items()),columns = ['term','tf'])
        df_kamusdata_negnya["idf"] = pd.DataFrame(list(IDF_neg.values()),columns = ['idf'])
        df_kamusdata_negnya["tf_idf_dict"] = df_kamusdata_negnya.tf * df_kamusdata_negnya.idf
        # print(df_kamusdata_negnya.head())
        # df_kamusdata_negnya["TF-IDF_dict"].sum()
    else:
        msg = 'Dataset masih Kosong!'

    if request.method == 'POST':
        response = request.form.get('sentimen')
        response = re.sub(r'[^\w]', ' ', remove_number(response))

    if len(response.strip()) == 0:
        return render_template("index_sentimen.html", score_pos="kosong", score_net="kosong", score_neg="kosong", prediksi_nb="kosong", prediksi="kosong", sentimen=False)
    elif len(response) != 0:
        
        sentimen=None
        dictme=None
        dictme = {'username':'input','term':response}
        sentimen = pd.DataFrame([dictme])
        pre=text_preprocesing(sentimen['term'], "satuan") # <--- satuan
        unique_term = list(set(pre[0]))

        if len(unique_term) == 0:
                return render_template("index_sentimen.html", score_pos="kosong", score_net="kosong", score_neg="kosong", prediksi_nb="kosong", prediksi="kosong", sentimen=False)

        prediksi_nb=naive_bayes_once("satuan", unique_term, df_kamusdata_posnya , df_kamusdata_negnya, df_kamusdata_netnya, df_kamusdata)
        score_posneg = score_probnb(prediksi_nb, n_dok.sum(), n_dok[1], n_dok[2], n_dok[0])

        if score_posneg["Negatif"] > score_posneg["Positif"] and score_posneg["Negatif"] > score_posneg["Netral"]:
            label_prediksi = "Negatif"
        elif score_posneg["Positif"] > score_posneg["Negatif"] and score_posneg["Positif"] > score_posneg["Netral"]:
            label_prediksi = "Positif"
        elif score_posneg["Netral"] > score_posneg["Negatif"] and score_posneg["Netral"] > score_posneg["Positif"]:
            label_prediksi = "Netral"
        else:
            label_prediksi = "Sentimen tidak diketahui"

 
        if add_loghistory(response, label_prediksi):
            return render_template("index_sentimen.html", score_pos=score_posneg["Positif"], score_net=score_posneg["Netral"], score_neg=score_posneg["Negatif"], prediksi_nb=[prediksi_nb.to_html(classes='table', header="true")], prediksi=label_prediksi, sentimen=' '.join(map(str, sentimen['term'].values)))


    return render_template('index.html')

def add_loghistory(sentimen, hasil_label):
    conn = mysql.connect()
    cursor = conn.cursor()

    # Insert data_preproces
    query = """INSERT INTO
                log_riwayat_testing (
                    datetime,
                    sentimen,
                    hasil_sentimen)
            VALUES(%s, %s, %s);"""

    now=datetime.now().strftime('%Y-%m-%d %H-%M-%S')

    cursor.execute(query, (now, sentimen, hasil_label))
    conn.commit()
    conn.close()
    return True

# route untuk halaman history
@app.route('/history', methods=['GET'])
def history():
    _data_history = load_riwayat_testing()
    if _data_history:
        return render_template('log_history.html', datahistory=_data_history, msg="sukses")
    else:
        return render_template('log_history.html', msg="Gagal memuat riwayat")

def load_riwayat_testing():
    query = """SELECT id, datetime, sentimen, hasil_sentimen, validasi_sentimen FROM log_riwayat_testing"""
    cursor = mysql.get_db().cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    cursor.close()
    return data if data else False

# route untuk update log_history untuk (validasi label)
@app.route('/history/edit', methods=['POST'])
def edit_history():
    id = request.form.get('my_id')
    validasi_sentimen = request.form.get('option')
    try:
        query = "UPDATE log_riwayat_testing SET validasi_sentimen = %s WHERE id = %s"
        conn = mysql.connect()
        cursor = conn.cursor()
        cursor.execute(query, (validasi_sentimen, int(id)))
        conn.commit()
        conn.close()
        msg = "sukses"
    except Exception as e:
        print(e)
        msg = "Gagal memperbarui riwayat"
    return redirect(url_for('history', msg=msg))

# route untuk clear semua loghistory
@app.route('/history/clear-log', methods=['POST'])
def clear_loghistory():
    query = """TRUNCATE TABLE log_riwayat_testing"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    conn.commit()
    conn.close()
    return redirect(url_for('history', msg="sukses"))

# route untuk halaman dataset
@app.route('/dataset')
def dataset():

        _dr=load_dataset_raw()
        _dp=load_dataset_processed()

        if (_dr.empty == True or _dp.empty == True):
            return render_template('dataset.html', )
        else:
            return render_template('dataset.html', dr=_dr.to_html(classes="table"), dp=_dp.to_html(classes="table"), msg="sukses")

# route untuk halaman upload dataset (.xlsx)
@app.route('/inputdataset', methods=['GET','POST'])
def input_dataset():
        form = docup()
        msg = ''
          
        if form.validate_on_submit():
            d = form.document.data
            docname = secure_filename(d.filename)
            d.save(os.path.join(app.config['UPLOAD_FOLDER'], docname))
            print("upload sukses")
            
            data_path = os.path.join(app.config['UPLOAD_FOLDER'], docname)
            ef = pd.read_excel(data_path)

            # Mengekstrak tweets dan labels dari DataFrame
            if 'tweets' in ef.columns and 'label' in ef.columns:
                tweets = ef['tweets'].tolist()
                labels = ef['label'].tolist()
            else:
            # Kolom tweets atau label tidak ditemukan
                msg = "Kolom 'tweets' dan 'label' Tidak Ditemukan Pada File Excel!"
                flash(msg, 'error')
                return render_template('dataset_upload.html', form=form, msg=msg)

            filename_raw = pd.DataFrame({'tweets': tweets, 'label': labels})

            if(load_kedb(filename_raw, len(filename_raw)) == "sukses"):
                filename_raw['label'] = filename_raw['label'].replace({'negatif': 0, 'positif':1, 'netral':2})
                # processed_data = text_preprocesing(filename_raw, "data-450")
                # filename_raw["tweets_case"] = filename_raw["tweets_case"]
                # filename_raw["tweets_tokenizing"] = filename_raw["tweets_tokenizing"]
                # filename_raw["tweet_normalized"] = filename_raw["tweet_normalized"]
                # filename_raw["tweet_tokens_WSW"] = filename_raw["tweet_tokens_WSW"]
                # filename_raw["tweet_tokens_stemmed"] = processed_data["tweet_tokens_stemmed"]
                filename_raw["tweet_tokens_stemmed"] = text_preprocesing(filename_raw, "data-520")
            
                filename_raw["tweet_list"] = filename_raw["tweet_tokens_stemmed"].astype(str).apply(convert_text_list)
                
                # Split Data
                train = filename_raw.sample(frac=0.8, random_state=100)
                test = filename_raw[~filename_raw.index.isin(train.index)]
                
                # Pembobotan Kata TF-IDF
                train["TF_dict"] = train['tweet_list'].apply(calc_TF)
                DF = calc_DF(train["TF_dict"])
                n_document = len(train)
                IDF = calc_IDF(n_document, DF)
                df_bobotdata = pd.DataFrame(list(IDF.items()),columns = ['term','idf'])
                df_bobotdata['df'] = pd.DataFrame(list(DF.values()), columns = ['df'])
                gk = train.groupby('label')

                # TF-IDF Positif
                df_bobotdata_pos = gk.get_group(1)
                DF_pos = calc_DF(df_bobotdata_pos["TF_dict"])
                IDF_pos = calc_IDF(n_document, DF_pos)
                df_bobotdata_posnya = pd.DataFrame(list(DF_pos.items()),columns = ['term','tf'])
                df_bobotdata_posnya["idf"] = pd.DataFrame(list(IDF_pos.values()),columns = ['idf'])
                df_bobotdata_posnya["TF-IDF_dict"] = df_bobotdata_posnya.tf * df_bobotdata_posnya.idf

                # TF-IDF Negatif
                df_bobotdata_neg = gk.get_group(0)
                DF_neg = calc_DF(df_bobotdata_neg["TF_dict"])
                IDF_neg = calc_IDF(n_document, DF_neg)
                df_bobotdata_negnya = pd.DataFrame(list(DF_neg.items()),columns = ['term','tf'])
                df_bobotdata_negnya["idf"] = pd.DataFrame(list(IDF_neg.values()),columns = ['idf'])
                df_bobotdata_negnya["TF-IDF_dict"] = df_bobotdata_negnya.tf * df_bobotdata_negnya.idf

                # TF-IDF Netral
                df_bobotdata_net = gk.get_group(2)
                DF_net = calc_DF(df_bobotdata_net["TF_dict"])
                IDF_net = calc_IDF(n_document, DF_net)
                df_bobotdata_netnya = pd.DataFrame(list(DF_net.items()),columns = ['term','tf'])
                df_bobotdata_netnya["idf"] = pd.DataFrame(list(IDF_net.values()),columns = ['idf'])
                df_bobotdata_netnya["TF-IDF_dict"] = df_bobotdata_netnya.tf * df_bobotdata_netnya.idf

                pre_loaddata('dataset_procesed')
                pre_loaddata('dataset_training')
                pre_loaddata('dataset_tes')
                pre_loaddata('bobot_data_all')
                pre_loaddata('bobot_positif')
                pre_loaddata('bobot_negatif')
                pre_loaddata('bobot_netral')

                conn = mysql.connect()
                cursor = conn.cursor()

                query = """INSERT INTO
                            dataset_procesed (
                                label,
                                tweet_tokens_stemmed)
                        VALUES(%s, "%s");"""

                i=0
                while i < len(filename_raw):
                    cursor.execute(query, (filename_raw['label'].iloc[i], filename_raw['tweet_tokens_stemmed'].iloc[i]))
                    conn.commit()
                    i+=1

                query = """INSERT INTO
                            dataset_tes (
                                label,
                                tweet_tokens_stemmed)
                        VALUES(%s, "%s");"""

                i=0
                while i < len(test):
                    cursor.execute(query, (test['label'].iloc[i], test['tweet_list'].iloc[i]))
                    conn.commit()
                    i+=1

                query = """INSERT INTO
                            dataset_training (
                                label,
                                tweet_tokens_stemmed)
                        VALUES(%s, "%s");"""

                i=0
                while i < len(train):
                    cursor.execute(query, (train['label'].iloc[i], train['tweet_list'].iloc[i]))
                    conn.commit()
                    i+=1

                query = """INSERT INTO
                            bobot_data_all (
                                term,
                                df,
                                idf)
                        VALUES(%s, %s, %s);"""

                i=0
                while i < len(df_bobotdata):
                    cursor.execute(query, ( df_bobotdata['term'][i], df_bobotdata['df'][i], df_bobotdata['idf'][i], ))
                    conn.commit()
                    i+=1

                query = """INSERT INTO
                            bobot_positif (
                                term,
                                tf,
                                idf,
                                tf_idf_dict)
                        VALUES(%s, %s ,%s ,%s);"""

                i=0
                while i < len(df_bobotdata_posnya):
                    cursor.execute(query, (df_bobotdata_posnya['term'][i], df_bobotdata_posnya['tf'][i], df_bobotdata_posnya['idf'][i], df_bobotdata_posnya['TF-IDF_dict'][i], ))
                    conn.commit()
                    i+=1

                query = """INSERT INTO
                            bobot_negatif (
                                term,
                                tf,
                                idf,
                                tf_idf_dict)
                        VALUES(%s ,%s ,%s,%s);"""

                i=0
                while i < len(df_bobotdata_negnya):
                    cursor.execute(query, (df_bobotdata_negnya['term'][i], df_bobotdata_negnya['tf'][i], df_bobotdata_negnya['idf'][i], df_bobotdata_negnya['TF-IDF_dict'][i], ))
                    conn.commit()
                    i+=1

                query = """INSERT INTO
                            bobot_netral (
                                term,
                                tf,
                                idf,
                                tf_idf_dict)
                        VALUES(%s ,%s ,%s,%s);"""

                i=0
                while i < len(df_bobotdata_netnya):
                    cursor.execute(query, (df_bobotdata_netnya['term'][i], df_bobotdata_netnya['tf'][i], df_bobotdata_netnya['idf'][i], df_bobotdata_netnya['TF-IDF_dict'][i], ))
                    conn.commit()
                    i+=1

                conn.close()

                msg = "Data berhasil ditambahkan.."
            return redirect(url_for('dataset', msg=msg))

        return render_template('dataset_upload.html', form=form, msg=msg)

def load_kedb(data, jumTot):
    temp_dat=pd.DataFrame(data, columns=["tweets","label"])
    dataset_raw='dataset_raw'
    if(pre_loaddata(dataset_raw)==True):
        query = """INSERT INTO
                    dataset_raw (
                        tweets,
                        label)
                VALUES (%s,%s);"""

        conn = mysql.connect()
        cursor = conn.cursor()

        i=0
        while i < jumTot:
            cursor.execute(query, (str(temp_dat['tweets'][i]), temp_dat['label'][i]))
            conn.commit()
            i+=1

        conn.close()
        return "sukses"
    else:
        return "gagal"

def load_dataset_raw():
    query = """SELECT tweets, label FROM dataset_raw"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['tweets','label'])
        df.index = df.index + 1
        return df
    else:
        return False

# Menampilkan Dataset Setelah Preprocessing
def load_dataset_processed():
    query = """SELECT tweet_tokens_stemmed, label FROM dataset_procesed"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        # df = pd.DataFrame(data, columns=['label','tweets_case','tweets_tokenizing','tweet_normalized','tweet_tokens_WSW','tweet_tokens_stemmed'])
        # df = df.rename(columns={'tweets_case': 'Case Folding', 'tweets_tokenizing': 'Tokenization', 'tweet_normalized': 'Normalization', 
        #                         'tweet_tokens_WSW': 'Stopwords', 'tweet_tokens_stemmed': 'Stemming'})
        df = pd.DataFrame(data, columns=['label','tweet_tokens_stemmed'])
        df = df.rename(columns={'tweet_tokens_stemmed': 'Stemming'})
        df.index = df.index + 1
        return df
    else:
        return False

# route untuk download template
@app.route('/download_template')
def download_template():
    template_path = 'template.xlsx'
    return send_file(template_path, as_attachment=True)

# route untuk halaman pembobotan
@app.route('/pembobotan', methods=['GET','POST'])
def bobotdata():
        
        msg=''

        _df_bobotdata = load_bobot_semua()
        _df_bobotdata_posnya = load_bobot_positif()
        _df_bobotdata_negnya = load_bobot_negatif()
        _df_bobotdata_netnya = load_bobot_netral()

        msg='bobot done..'

        if ( _df_bobotdata.empty == True or _df_bobotdata_posnya.empty == True or _df_bobotdata_negnya.empty == True or _df_bobotdata_netnya.empty == True):
            return render_template('pembobotan.html', )

        else:
            return render_template('pembobotan.html',
                                    df_bobotdata= _df_bobotdata.to_html(classes="table"),
                                    df_bobotdata_pos= _df_bobotdata_posnya.to_html(classes="table"),
                                    df_bobotdata_neg= _df_bobotdata_negnya.to_html(classes="table"),
                                    df_bobotdata_net= _df_bobotdata_netnya.to_html(classes="table"),
                                    msg=msg)

def load_bobot_semua():
    query = """SELECT term, df, idf FROM bobot_data_all"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','df','idf'])
        df.index = df.index + 1
        return df
    else:
        return False

def load_bobot_positif():
    query = """SELECT term, tf, idf, tf_idf_dict FROM bobot_positif"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','tf','idf','tf_idf_dict'])
        df.index = df.index + 1
        return df
    else:
        return False

def load_bobot_negatif():
    query = """SELECT term, tf, idf, tf_idf_dict FROM bobot_negatif"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','tf','idf','tf_idf_dict'])
        df.index = df.index + 1
        return df
    else:
        return False
    
def load_bobot_netral():
    query = """SELECT term, tf, idf, tf_idf_dict FROM bobot_netral"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','tf','idf','tf_idf_dict'])
        df.index = df.index + 1
        return df
    else:
        return False

# route untuk halaman data training dan testing
@app.route('/train-testing', methods=['GET','POST'])
def train_tes():
        
        _train = load_training()
        _test = load_testing()

        msg="traintest done.."

        if ( _train.empty == True or _test.empty == True ):
            return render_template('train_tes.html', )

        else:
            return render_template('train_tes.html',
                                    train= _train.to_html(classes="table"),
                                    test= _test.to_html(classes="table"),
                                    msg=msg)

def load_training():
    query = """SELECT label, tweet_tokens_stemmed as tweet_list FROM dataset_training"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['label','tweet_list'])
        df.index = df.index + 1
        return df
    else:
        return False

def load_testing():
    query = """SELECT label, tweet_tokens_stemmed as tweet_list FROM dataset_tes"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['label','tweet_list'])
        df.index = df.index + 1
        return df
    else:
        return False

# route untuk halaman uji testing
@app.route('/analisis-hasil', methods=['GET','POST'])
def testing():

        msg=""

        # load data
        _dataset = load_dataset_processed()
        _train = load_train()
        _test = load_tes()
        _bobotdata = load_bobot_all()
        _bobotdata_pos = load_bobot_pos()
        _bobotdata_neg = load_bobot_neg()
        _bobotdata_net = load_bobot_net()

        if (_dataset.empty == True or _train.empty == True or _test.empty == True or _bobotdata.empty == True or _bobotdata_neg.empty == True or _bobotdata_pos.empty == True or _bobotdata_net.empty == True):
            return render_template('uji_testing.html', )

        else:
            dataset = pd.DataFrame(_dataset, columns=["label"])
            train = pd.DataFrame(_train, columns=["label","tweet_list"])
            test = pd.DataFrame(_test, columns=["label","tweet_list"])
            df_bobotdata = pd.DataFrame(_bobotdata, columns=["term","df","idf"])
            df_bobotdata_posnya = pd.DataFrame(_bobotdata_pos, columns=['term','tf','idf','tf_idf_dict'])
            df_bobotdata_netnya = pd.DataFrame(_bobotdata_net, columns=['term','tf','idf','tf_idf_dict'])
            df_bobotdata_negnya = pd.DataFrame(_bobotdata_neg, columns=['term','tf','idf','tf_idf_dict'])

            datahasil = {}
            datahasil = naive_bayes(test, train, df_bobotdata_posnya, df_bobotdata_negnya, df_bobotdata_netnya, df_bobotdata)

            show = pd.DataFrame(datahasil)
            show.reset_index(drop=True, inplace=True) 
            show.index = show.index + 1 

            pd.set_option('future.no_silent_downcasting', True)
            show['Label_Prediksi'] = show['Label_Prediksi'].replace({'Positif': 1, 'Netral':2, 'Negatif':0})

            show["label_asal"]=[x for x in test["label"]]

            df_matrik = pd.DataFrame(show, columns=['label_asal','Label_Prediksi'])

            confusion_matrix = pd.crosstab(df_matrik['label_asal'], df_matrik['Label_Prediksi'], rownames=['Actual'], colnames=['Predicted'])

            # buat visualisasi data
            diagram_dataset_pie=piechart(dataset)
            kemunculan_kata=barchart(df_bobotdata[["term","df"]])

            word_cloud_pos=get_wordcloud(dict(df_bobotdata_posnya[['term','tf_idf_dict']].values))
            word_cloud_neg=get_wordcloud(dict(df_bobotdata_negnya[['term','tf_idf_dict']].values))
            word_cloud_net=get_wordcloud(dict(df_bobotdata_netnya[['term','tf_idf_dict']].values))

            msg="test done.."

            return render_template('analisis_hasil.html',
                                    # Hasil Posterior
                                    show=show.to_html(classes="table"),
                                    # Confusion Matrix
                                    confusion_matrix=confusion_matrix.to_html(classes="table"),
                                    # Perhitungan Confusion Matrix
                                    akurasi= round(akurasi(confusion_matrix) * 100, 2),
                                    presisi_pos = round(presisi_pos(confusion_matrix) * 100, 2),
                                    presisi_net = round(presisi_net(confusion_matrix) * 100, 2),
                                    presisi_neg = round(presisi_neg(confusion_matrix) * 100, 2),
                                    rata_rata_presisi = round(rata_rata_presisi(confusion_matrix) * 100, 2),
                                    recall_pos = round(recall_pos(confusion_matrix) * 100, 2),
                                    recall_neg = round(recall_neg(confusion_matrix) * 100, 2),
                                    recall_net = round(recall_net(confusion_matrix) * 100, 2),
                                    rata_rata_recall = round(rata_rata_recall(confusion_matrix) * 100, 2),
                                    Fscore_pos = round(Fscore_pos(confusion_matrix) * 100, 2),
                                    Fscore_neg = round(Fscore_neg(confusion_matrix) * 100, 2),
                                    Fscore_net = round(Fscore_net(confusion_matrix) * 100, 2),
                                    rata_rata_Fscore = round(rata_rata_Fscore(confusion_matrix) * 100, 2),
                                    # Diagram Batang
                                    barJSON=kemunculan_kata,
                                    # Diagram Lingkaran
                                    pieJSON=diagram_dataset_pie,
                                    # Wordcloud
                                    img_data_pos=word_cloud_pos,
                                    img_data_net=word_cloud_net,
                                    img_data_neg=word_cloud_neg,
                                    msg=msg)

def load_dataset_proces():
    query = """SELECT label FROM dataset_procesed"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['label'])
        return df
    else:
        return False

def load_train():
    query = """SELECT label, tweet_tokens_stemmed as tweet_list FROM dataset_training"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['label','tweet_list'])
        return df
    else:
        return False

def load_tes():
    query = """SELECT label, tweet_tokens_stemmed as tweet_list FROM dataset_tes"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['label','tweet_list'])
        return df
    else:
        return False

def load_bobot_all():
    query = """SELECT term, df, idf FROM bobot_data_all"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','df','idf'])
        return df
    else:
        return False

def load_bobot_pos():
    query = """SELECT term, tf, idf, tf_idf_dict FROM bobot_positif"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','tf','idf','tf_idf_dict'])
        return df
    else:
        return False

def load_bobot_neg():
    query = """SELECT term, tf, idf, tf_idf_dict FROM bobot_negatif"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','tf','idf','tf_idf_dict'])
        return df
    else:
        return False
    
def load_bobot_net():
    query = """SELECT term, tf, idf, tf_idf_dict FROM bobot_netral"""
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    data = cursor.fetchall()
    conn.close()
    if data:
        df = pd.DataFrame(data, columns=['term','tf','idf','tf_idf_dict'])
        return df
    else:
        return False

def pre_loaddata(table):
    query = "DELETE FROM `{}`;".format(table)
    query_reset_id = "TRUNCATE TABLE `{}`;".format(table)
    conn = mysql.connect()
    cursor = conn.cursor()
    cursor.execute(query)
    conn.commit()
    cursor.execute(query_reset_id)
    conn.commit()
    conn.close()
    return True


if __name__ == '__main__':
    app.run(debug=True)
