import pandas as pd
import numpy as np
from tf_idf import *
from preprocesing_text import select_unique as unique


# perhitungan akurasi menggunakan confusion_matrix
def akurasi(confusion_matrix):
    # Menghitung total prediksi yang benar (diagonal dari confusion matrix)
    total_prediksi_benar = confusion_matrix.iloc[0, 0] + confusion_matrix.iloc[1, 1] + confusion_matrix.iloc[2, 2]

    # Menghitung total jumlah prediksi untuk semua kelas
    total_prediksi = confusion_matrix.values.sum()

    # Menghitung akurasi
    return total_prediksi_benar / total_prediksi

#Menghitung Presisi Positif
def presisi_pos(confusion_matrix):
    true_positive = confusion_matrix.iloc[1, 1]
    false_positive = confusion_matrix.iloc[1, 0] + confusion_matrix.iloc[1, 2]

    precision = true_positive / (true_positive + false_positive)

    return precision

#Menghitung Presisi Netral
def presisi_net(confusion_matrix):
    true_neutral = confusion_matrix.iloc[2, 2]
    false_neutral = confusion_matrix.iloc[2, 0] + confusion_matrix.iloc[2, 1]

    precision = true_neutral / (true_neutral + false_neutral)

    return precision

#Menghitung Presisi Negatif
def presisi_neg(confusion_matrix):
    true_negative = confusion_matrix.iloc[0, 0]
    false_negative = confusion_matrix.iloc[0, 1] + confusion_matrix.iloc[0, 2]

    precision = true_negative / (true_negative + false_negative)

    return precision

def rata_rata_presisi(confusion_matrix):
    presisi_positif = presisi_pos(confusion_matrix)
    presisi_netral = presisi_net(confusion_matrix)
    presisi_negatif = presisi_neg(confusion_matrix)

    rata_rata = (presisi_positif + presisi_netral + presisi_negatif) / 3

    return rata_rata

#Menghitung Recall Positif
def recall_pos(confusion_matrix):
    true_positive = confusion_matrix.iloc[1, 1]
    false_negative = confusion_matrix.iloc[0, 1] + confusion_matrix.iloc[0, 2]
    false_neutral = confusion_matrix.iloc[2, 0] + confusion_matrix.iloc[2, 1]

    recall = true_positive / (true_positive + false_negative + false_neutral)

    return recall

#Menghitung Recall Netral
def recall_net(confusion_matrix):
    true_neutral = confusion_matrix.iloc[2, 2]
    false_negative = confusion_matrix.iloc[0, 1] + confusion_matrix.iloc[0, 2]
    false_positive = confusion_matrix.iloc[1, 0] + confusion_matrix.iloc[1, 2]

    recall = true_neutral / (true_neutral + false_negative + false_positive)

    return recall

#Menghitung Recall Negatif
def recall_neg(confusion_matrix):
    true_negative = confusion_matrix.iloc[0, 0]
    false_neutral = confusion_matrix.iloc[2, 1] + confusion_matrix.iloc[2, 0]
    false_positive = confusion_matrix.iloc[1, 0] + confusion_matrix.iloc[1, 2]

    recall = true_negative / (true_negative + false_neutral + false_positive)

    return recall

def rata_rata_recall(confusion_matrix):
    recall_positif = recall_pos(confusion_matrix)
    recall_netral = recall_net(confusion_matrix)
    recall_negatif = recall_neg(confusion_matrix)

    rata_rata = (recall_positif + recall_netral + recall_negatif) / 3

    return rata_rata

#Menghitung F1-Score Positif
def Fscore_pos(confusion_matrix):
    true_positive = confusion_matrix.iloc[1, 1]
    false_positive = confusion_matrix.iloc[1, 0] + confusion_matrix.iloc[1, 2]
    false_negative = confusion_matrix.iloc[0, 1] + confusion_matrix.iloc[0, 2]
    false_neutral = confusion_matrix.iloc[2, 0] + confusion_matrix.iloc[2, 1]

    precision = true_positive / (true_positive + false_positive)
    recall = true_positive / (true_positive + false_negative + false_neutral)

    f1_score = 2 * (precision * recall) / (precision + recall) if precision + recall > 0 else 0

    return f1_score

#Menghitung F1-Score Netral
def Fscore_net(confusion_matrix):
    true_neutral = confusion_matrix.iloc[2, 2]
    false_positive = confusion_matrix.iloc[1, 0] + confusion_matrix.iloc[1, 2]
    false_negative = confusion_matrix.iloc[0, 1] + confusion_matrix.iloc[0, 2]
    false_neutral = confusion_matrix.iloc[2, 0] + confusion_matrix.iloc[2, 1]

    precision = true_neutral / (true_neutral + false_neutral)
    recall = true_neutral / (true_neutral + false_negative + false_positive)

    f1_score = 2 * (precision * recall) / (precision + recall) if precision + recall > 0 else 0

    return f1_score

#Menghitung F1-Score Negatif
def Fscore_neg(confusion_matrix):
    true_negative = confusion_matrix.iloc[0, 0]
    false_positive = confusion_matrix.iloc[1, 0] + confusion_matrix.iloc[1, 2]
    false_negative = confusion_matrix.iloc[0, 1] + confusion_matrix.iloc[0, 2]
    false_neutral = confusion_matrix.iloc[2, 0] + confusion_matrix.iloc[2, 1]

    precision = true_negative / (true_negative + false_negative)
    recall = true_negative / (true_negative + false_positive + false_neutral)

    f1_score = 2 * (precision * recall) / (precision + recall) if precision + recall > 0 else 0

    return f1_score

def rata_rata_Fscore(confusion_matrix):
    Fscore_positif = Fscore_pos(confusion_matrix)
    Fscore_netral = Fscore_net(confusion_matrix)
    Fscore_negatif = Fscore_neg(confusion_matrix)

    rata_rata = (Fscore_positif + Fscore_netral + Fscore_negatif) / 3

    return rata_rata

# hitung likelihood probability
def hitung_bayes(found,n_tfidf,n_totaltfidf,total_idf):
    if found == "true":
        return ((n_tfidf+1) / ((n_totaltfidf)+total_idf))
    elif found == "false":
        return ( 1 / ((n_totaltfidf)+total_idf))

# perhitungan prior dan posterior probability
def score_probnb(data, n_dok, n_jumpos, n_jumneg, n_jumnet):
    # perkalian semua score pada kelas di dataframe
    score_tot_pos =  np.prod(np.array(data["positif"]))
    score_tot_net =  np.prod(np.array(data["netral"]))
    score_tot_neg =  np.prod(np.array(data["negatif"]))

    # hasil variabel skor disini merupakan hasil dari perhitungan posterior probability
    score={
        "Positif": (n_jumpos/n_dok) * score_tot_pos,
        "Netral": (n_jumnet/n_dok) * score_tot_net,
        "Negatif": (n_jumneg/n_dok) * score_tot_neg,
        
    }
    return score

def naive_bayes(test, train, bobot_pos, bobot_neg, bobot_net, idf_all):
    print("proses naive bayes (many)")

    # data train
    df_train = pd.DataFrame(train, columns=["label","tweet_list"])

    # data tes
    df_files = pd.DataFrame(test, columns=["label","tweet_list"])

    df_files['tweet_list'] = df_files['tweet_list'].apply(ast.literal_eval).tolist()
    df_files['unique_term'] = df_files['tweet_list'].apply(unique)

    ds_bobot_pos = pd.DataFrame(bobot_pos, columns=["term", "tf_idf_dict"])
    ds_bobot_net = pd.DataFrame(bobot_net, columns=["term", "tf_idf_dict"])
    ds_bobot_neg = pd.DataFrame(bobot_neg, columns=["term", "tf_idf_dict"])
    ds_bobot_all = pd.DataFrame(idf_all, columns=["term", "idf"])

    gk = df_train.groupby('label')
    n_dok = gk.size()

    n_dok, n_jumpos, n_jumneg, n_jumnet = n_dok.sum(), n_dok[1], n_dok[0], n_dok[2]

    datahasil={}

    x = 0
    while x < len(df_files):
        n_label_prediksi={}
        n_hasil_nb = naive_bayes_once("satuan", df_files['unique_term'].iloc[x], ds_bobot_pos, ds_bobot_neg, ds_bobot_net, ds_bobot_all)
        n_score_nb = score_probnb(n_hasil_nb, n_dok, n_jumpos, n_jumneg, n_jumnet)

        if n_score_nb["Negatif"] > n_score_nb["Positif"] and n_score_nb["Negatif"] > n_score_nb["Netral"]:
            n_label_prediksi[x] = "Negatif"
        elif n_score_nb["Positif"] > n_score_nb["Negatif"] and n_score_nb["Positif"] > n_score_nb["Netral"]:
            n_label_prediksi[x] = "Positif"
        elif n_score_nb["Netral"] > n_score_nb["Negatif"] and n_score_nb["Netral"] > n_score_nb["Positif"]:
            n_label_prediksi[x] = "Netral"
        else:
            n_label_prediksi[x] = "Sentimen Tidak Diketahui"

        datahasil[x] = {
            "Positif": n_score_nb["Positif"],
            "Netral": n_score_nb["Netral"],
            "Negatif": n_score_nb["Negatif"],
            "Label_Prediksi": n_label_prediksi[x]
        }

        n_hasil_nb = None
        n_score_nb = None
        n_label_prediksi = None
        x+=1

    datahasil_final=pd.DataFrame(columns=["Positif","Netral", "Negatif", "Label_Prediksi"])
    x=0
    while x < len(datahasil.keys()):
        datahasil_final.loc[x] = datahasil[x]["Positif"], datahasil[x]["Netral"], datahasil[x]["Negatif"], datahasil[x]["Label_Prediksi"]
        x+=1
    return datahasil_final

def naive_bayes_once(mode, files, bobot_pos, bobot_neg, bobot_net, idf_all):
    # read dataset
    df_files = pd.DataFrame(files, columns=["term"])
    ds_bobot_pos = pd.DataFrame(bobot_pos, columns=["term", "tf_idf_dict"])
    ds_bobot_neg = pd.DataFrame(bobot_neg, columns=["term", "tf_idf_dict"])
    ds_bobot_net = pd.DataFrame(bobot_net, columns=["term", "tf_idf_dict"])
    ds_bobot_all = pd.DataFrame(idf_all, columns=["term", "idf"])

    jml_idf_all = ds_bobot_all["idf"].sum()

    if mode == "satuan":
        positif, negatif, netral = {}, {}, {}

        for j,term in enumerate(df_files["term"]):
            # positif
            for i,kata in enumerate(ds_bobot_pos['term']):
                if term == kata:
                    positif[term] = hitung_bayes("true", ds_bobot_pos["tf_idf_dict"][i], ds_bobot_pos["tf_idf_dict"].sum(), jml_idf_all)
                    break
                elif term != kata :
                    positif[term] = hitung_bayes("false", 0, ds_bobot_pos["tf_idf_dict"].sum(), jml_idf_all)

            # negatif
            for i,kata in enumerate(ds_bobot_neg['term']):
                if term == kata:
                    negatif[term] = hitung_bayes("true", ds_bobot_neg["tf_idf_dict"][i], ds_bobot_neg["tf_idf_dict"].sum(), jml_idf_all)
                    break
                elif term != kata :
                    negatif[term] = hitung_bayes("false", 0, ds_bobot_neg["tf_idf_dict"].sum(), jml_idf_all)

            # netral
            for i,kata in enumerate(ds_bobot_net['term']):
                if term == kata:
                    netral[term] = hitung_bayes("true", ds_bobot_net["tf_idf_dict"][i], ds_bobot_net["tf_idf_dict"].sum(), jml_idf_all)
                    break
                elif term != kata :
                    netral[term] = hitung_bayes("false", 0, ds_bobot_net["tf_idf_dict"].sum(), jml_idf_all)

        df_files["positif"] = df_files["term"].map(positif)
        df_files["netral"] = df_files["term"].map(netral)
        df_files["negatif"] = df_files["term"].map(negatif)

    else:
        print("data kosong!!")
        pass

    return df_files
