import pandas as pd
import nltk_config as nltkdr
import nltk
import string
import re
from Sastrawi.Stemmer.StemmerFactory import StemmerFactory
from Sastrawi.StopWordRemover.StopWordRemoverFactory import StopWordRemoverFactory
from nltk.tokenize import word_tokenize
from nltk.corpus import stopwords

nltk.download('punkt')

_npath = nltkdr._nltkdir()
# load nltk data from dir
nltk.data.path.append(_npath + '/resources/punkt')
nltk.data.path.append(_npath + '/resources/stopwords')

def select_unique(text):
    return pd.Series(text).unique()

# hapus unique simbol
def remove_tweet_special(text):
    # hapus tab, new line, ans back slice
    text = text.replace('\\t'," ").replace('\\n'," ").replace('\\u'," ").replace('\\',"")
    # hapus non ASCII (emoticon, chinese word, .etc)
    text = text.encode('ascii', 'replace').decode('ascii')
    # hapus mention, link, hashtag
    text = ' '.join(re.sub("([@#][A-Za-z0-9]+)|(\w+:\/\/\S+)", " ", text).split())
    # hapus incomplete URL
    text = re.sub('http[s]?://\S+', '', text)
    return text

#hapus number
def remove_number(text):
    return  re.sub(r"\d+", "", text)

#hapus punctuation
def remove_punctuation(text):
    return text.translate(str.maketrans("","",string.punctuation))

#hapus whitespace leading & trailing
def remove_whitespace_LT(text):
    return text.strip()

#remove multiple whitespace into single whitespace
def remove_whitespace_multiple(text):
    return re.sub('\s+',' ',text)

# remove single char
def remove_singl_char(text):
    return re.sub(r"\b[a-zA-Z]\b", "", text)

# Tokenizing
def word_tokenize_wrapper(text):
    return word_tokenize(text)

# Normalisasi kata
normalizad_word = pd.read_excel(_npath + "/normalisasi.xlsx")
normalizad_word_dict = {}

for index, row in normalizad_word.iterrows():
    if row.iloc[0] not in normalizad_word_dict:
        normalizad_word_dict[row.iloc[0]] = row.iloc[1]

def normalized_term(document):
    return [normalizad_word_dict[term] if term in normalizad_word_dict else term for term in document]

# Remove stopword
list_stopwords = stopwords.words('indonesian')

factorysw = StopWordRemoverFactory()
list_stopwords.extend(factorysw.get_stop_words())

list_stopwords.extend(['yg', 'dg', 'rt', 'dgn', 'ny', 'd', 'klo',
                       'kalo', 'amp', 'biar', 'bikin', 'bilang',
                       'gak', 'ga', 'krn', 'nya', 'nih', 'sih',
                       'si', 'tau', 'tdk', 'tuh', 'utk', 'ya',
                       'jd', 'jgn', 'sdh', 'aja', 'n', 't', 'p', 'ak',
                       'nyg', 'hehe', 'pen', 'u', 'nan', 'loh', 'rt',
                       '&amp', 'yah'])

txt_stopword = pd.read_csv(_npath + "/stopwords.csv", names=["stopwords"], header=None)
list_stopwords.extend(txt_stopword["stopwords"][0].split(' '))

list_stopwords = set(list_stopwords)

def stopwords_removal(words):
    return [word for word in words if word not in list_stopwords]

# Steamming kata

# create stemmer
factory = StemmerFactory()
stemmer = factory.create_stemmer()

# stemmed
def stemmed_wrapper(term):
    return stemmer.stem(term)

term_dict = {}

def normalize_documents_sastrawi(dataset):
    term_dict = {}

    for document in dataset:
        # term_dict = { term for term in document if term not in term_dict term_dict[term] = ' ' }
        for term in document:
            if term not in term_dict:
                term_dict[term] = ' '

    for term in term_dict:
        term_dict[term] = stemmed_wrapper(term)

    return term_dict

# Preprosesing menggunakan fungsi yang ada
# def text_preprocessing(dataset, mode):
#     if mode == "satuan":
#         Twit_data = pd.DataFrame(list(dataset.items()), columns=['tweets'])
#     else:
#         Twit_data = pd.DataFrame(dataset, columns=['label', 'tweets'])

#     Twit_data['tweets'] = Twit_data['tweets'].astype(str)
    
#     # Case Folding
#     Twit_data['tweets_case'] = Twit_data['tweets'].str.lower()
    
#     # Data Cleaning
#     Twit_data['tweets'] = Twit_data['tweets_case'].apply(remove_tweet_special)
#     Twit_data['tweets'] = Twit_data['tweets_case'].apply(remove_number)
#     Twit_data['tweets'] = Twit_data['tweets_case'].apply(remove_punctuation)
#     Twit_data['tweets'] = Twit_data['tweets_case'].apply(remove_whitespace_LT)
#     Twit_data['tweets'] = Twit_data['tweets_case'].apply(remove_whitespace_multiple)
#     Twit_data['tweets'] = Twit_data['tweets_case'].apply(remove_singl_char)
    
#     # Tokenizing
#     Twit_data['tweets_tokenizing'] = Twit_data['tweets_case'].apply(word_tokenize_wrapper)

#     # Normalisasi kata
#     Twit_data['tweet_normalized'] = Twit_data['tweets_tokenizing'].apply(normalized_term)

#     # Remove stopword
#     Twit_data['tweet_tokens_WSW'] = Twit_data['tweet_normalized'].apply(stopwords_removal)

#     # Stemming kata
#     Twit_data['tweet_tokens_stemmed'] = Twit_data['tweet_tokens_WSW'].apply(stemmed_wrapper)

    # return Twit_data[['tweets_case', 'tweets_tokenizing', 'tweet_normalized', 'tweet_tokens_WSW', 'tweet_tokens_stemmed']]
